package fr.inria.phoenix.diasuite.framework.context.averagetemperature;

import java.io.Serializable;

/**
 * An object to store a value published by the context <code>AverageTemperature</code>.
 *
 * <pre>
 * context AverageTemperature as Float indexed by location as String {
 * 	when provided temperature from TemperatureSensor
 * 	get temperature from TemperatureSensor
 * 	always publish;
 * }
 * </pre>
 */
public final class AverageTemperatureValue implements Serializable {
    private static final long serialVersionUID = 0;
    
    private java.lang.Float value;
    
    /**
     * Get the value of the context <code>AverageTemperature</code>
     * 
     * @return the value of the context <code>AverageTemperature</code>
     */
    public java.lang.Float value() {
        return value;
    }
    
    private AverageTemperatureIndices indices;
    
    /**
     * Get the value of the indices of the context <code>AverageTemperature</code>
     * 
     * @return the value of the indices
     */
    public AverageTemperatureIndices indices() {
        return indices;
    }
    
    public AverageTemperatureValue(java.lang.Float value, AverageTemperatureIndices indices) {
        this.value = value;
        this.indices = indices;
    }
    
    public AverageTemperatureValue(java.lang.Float value,
            java.lang.String location) {
        this.value = value;
        this.indices = new AverageTemperatureIndices(location);
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        result = prime * result + ((indices == null) ? 0 : indices.hashCode());
        return result;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        AverageTemperatureValue other = (AverageTemperatureValue) obj;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        if (indices == null) {
            if (other.indices != null)
                return false;
        } else if (!indices.equals(other.indices))
            return false;
        return true;
    }
}
