package fr.inria.phoenix.diasuite.framework.context.hvacregulation;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.averagetemperature.AverageTemperatureValue;

/**
 * <pre>
 * context HVACRegulation as TemperatureLevel indexed by location as String {
 * 	when provided AverageTemperature 
 * 	get targetTemperature from ControlPanel,
 * 		status from HVACSystem
 * 	maybe publish;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractHVACRegulation extends Service {
    
    public AbstractHVACRegulation(ServiceConfiguration serviceConfiguration) {
        super("/Context/HVACRegulation/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("averageTemperature", "/Context/AverageTemperature/"); // subscribe to AverageTemperature context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("averageTemperature") && source.isCompatible("/Context/AverageTemperature/")) {
            AverageTemperatureValue averageTemperatureValue = new AverageTemperatureValue((java.lang.Float) value,
                    (java.lang.String) indexes[0]);
            
            HVACRegulationValuePublishable returnValue = onAverageTemperature(averageTemperatureValue, new DiscoverForAverageTemperature());
            if(returnValue != null && returnValue.doPublish()) {
                setHVACRegulation(returnValue.getValue(), returnValue.getLocation());
            }
        }
    }
    
    @Override
    public final Object getValueCalled(java.util.Map<String, Object> properties, RemoteServiceInfo source, String valueName,
            Object... indexes) throws Exception {
        if (valueName.equals("hVACRegulation")) {
            return getLastValue((java.lang.String) indexes[0]);
        }
        throw new InvocationException("Unsupported method call: " + valueName);
    }
    // End of methods from the Service class
    
    // Code relative to the return value of the context
    private java.util.Map<HVACRegulationIndices, fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel> contextValues = new java.util.HashMap<HVACRegulationIndices, fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel>();
    
    private void setHVACRegulation(fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel newContextValue, java.lang.String location) {
        HVACRegulationIndices indices = new HVACRegulationIndices(location);
        contextValues.put(indices, newContextValue);
        getProcessor().publishValue(getOutProperties(), "hVACRegulation", newContextValue, location);
    }
    
    /**
     * Get the last value of the context
     * 
     * @param location the index <code>location</code> for the last value
     * @return the latest value published by the context for the given indexes
     */
    protected final fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel getLastValue(java.lang.String location) {
        HVACRegulationIndices indices = new HVACRegulationIndices(location);
        return contextValues.get(indices);
    }
    
    /**
     * A class that represents a value that might be published for the <code>HVACRegulation</code> context. It is used by
     * event methods that might or might not publish values for this context.
     */
    protected final static class HVACRegulationValuePublishable {
        
        // The value of the context
        private fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel value;
        // The location index
        private java.lang.String location;
        // Whether the value should be published or not
        private boolean doPublish;
        
        public HVACRegulationValuePublishable(fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel value,
                java.lang.String location,
                boolean doPublish) {
            this.value = value;
            this.location = location;
            this.doPublish = doPublish;
        }
        
        /**
         * @return the value of the context that might be published
         */
        public fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel getValue() {
            return value;
        }
        
        /**
         * Sets the value that might be published
         * 
         * @param value the value that will be published if {@link #doPublish()} returns true
         */
        public void setValue(fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel value) {
            this.value = value;
        }
        
        /**
         * @return the <code>location</code> index for the value of the context that might be published
         */
        public java.lang.String getLocation() {
            return location;
        }
        
        /**
         * Sets the <code>location</code> index for the value that might be published
         * 
         * @param location the <code>location</code> index
         */
        public void setLocation(java.lang.String location) {
            this.location = location;
        }
        
        /**
         * @return true if the value should be published
         */
        public boolean doPublish() {
            return doPublish;
        }
        
        /**
         * Set the value to be publishable or not
         * 
         * @param doPublish if true, the value will be published
         */
        public void setDoPublish(boolean doPublish) {
            this.doPublish = doPublish;
        }
    }
    // End of code relative to the return value of the context
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>AverageTemperature</code> context publishes a value.
     * 
     * <pre>
     * when provided AverageTemperature 
     * 	get targetTemperature from ControlPanel,
     * 		status from HVACSystem
     * 	maybe publish;
     * </pre>
     * 
     * @param averageTemperatureValue the value of the <code>AverageTemperature</code> context.
     * @param discover a discover object to get value from devices and contexts
     * @return a {@link HVACRegulationValuePublishable} that says if the context should publish a value and which value it should publish
     */
    protected abstract HVACRegulationValuePublishable onAverageTemperature(AverageTemperatureValue averageTemperatureValue, DiscoverForAverageTemperature discover);
    
    // End of interaction contract implementation
    
    // Discover object for AverageTemperature
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided AverageTemperature 
     * 	get targetTemperature from ControlPanel,
     * 		status from HVACSystem
     * 	maybe publish;
     * </code>
     */
    protected final class DiscoverForAverageTemperature {
        private final ControlPanelDiscovererForAverageTemperature controlPanelDiscoverer = new ControlPanelDiscovererForAverageTemperature(AbstractHVACRegulation.this);
        private final HVACSystemDiscovererForAverageTemperature hVACSystemDiscoverer = new HVACSystemDiscovererForAverageTemperature(AbstractHVACRegulation.this);
        
        /**
         * @return a {@link ControlPanelDiscovererForAverageTemperature} object to discover <code>ControlPanel</code> devices
         */
        public ControlPanelDiscovererForAverageTemperature controlPanels() {
            return controlPanelDiscoverer;
        }
        
        /**
         * @return a {@link HVACSystemDiscovererForAverageTemperature} object to discover <code>HVACSystem</code> devices
         */
        public HVACSystemDiscovererForAverageTemperature hVACSystems() {
            return hVACSystemDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>ControlPanel</code> devices to get their sources for the
     * <code>when provided AverageTemperature</code> interaction contract.
     * 
     * <pre>
     * device ControlPanel extends PhysicalDevice {
     * 	source targetTemperature as Float;
     * 	action SetTargetTemperature;
     * }
     * </pre>
     */
    protected final static class ControlPanelDiscovererForAverageTemperature {
        private Service serviceParent;
        
        private ControlPanelDiscovererForAverageTemperature(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private ControlPanelCompositeForAverageTemperature instantiateComposite() {
            return new ControlPanelCompositeForAverageTemperature(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>ControlPanel</code> devices
         * 
         * @return a {@link ControlPanelCompositeForAverageTemperature} object composed of all discoverable <code>ControlPanel</code>
         */
        public ControlPanelCompositeForAverageTemperature all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>ControlPanel</code> devices
         * 
         * @return a {@link ControlPanelProxyForAverageTemperature} object pointing to a random discoverable <code>ControlPanel</code> device
         */
        public ControlPanelProxyForAverageTemperature anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>ControlPanel</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link ControlPanelCompositeForAverageTemperature} object composed of all matching <code>ControlPanel</code> devices
         */
        public ControlPanelCompositeForAverageTemperature whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>ControlPanel</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link ControlPanelCompositeForAverageTemperature} object composed of all matching <code>ControlPanel</code> devices
         */
        public ControlPanelCompositeForAverageTemperature whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>ControlPanel</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link ControlPanelCompositeForAverageTemperature} object composed of all matching <code>ControlPanel</code> devices
         */
        public ControlPanelCompositeForAverageTemperature whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>ControlPanel</code> devices to get their sources for the
     * <code>when provided AverageTemperature</code> interaction contract.
     * 
     * <pre>
     * device ControlPanel extends PhysicalDevice {
     * 	source targetTemperature as Float;
     * 	action SetTargetTemperature;
     * }
     * </pre>
     */
    protected final static class ControlPanelCompositeForAverageTemperature extends fr.inria.diagen.core.service.composite.Composite<ControlPanelProxyForAverageTemperature> {
        private ControlPanelCompositeForAverageTemperature(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/ControlPanel/");
        }
        
        @Override
        protected ControlPanelProxyForAverageTemperature instantiateProxy(RemoteServiceInfo rsi) {
            return new ControlPanelProxyForAverageTemperature(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link ControlPanelCompositeForAverageTemperature}, filtered using the attribute <code>id</code>.
         */
        public ControlPanelCompositeForAverageTemperature andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link ControlPanelCompositeForAverageTemperature}, filtered using the attribute <code>location</code>.
         */
        public ControlPanelCompositeForAverageTemperature andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link ControlPanelCompositeForAverageTemperature}, filtered using the attribute <code>user</code>.
         */
        public ControlPanelCompositeForAverageTemperature andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
    }
    
    /**
     * A proxy to one <code>ControlPanel</code> device to get its sources for the
     * <code>when provided AverageTemperature</code> interaction contract.
     * 
     * <pre>
     * device ControlPanel extends PhysicalDevice {
     * 	source targetTemperature as Float;
     * 	action SetTargetTemperature;
     * }
     * </pre>
     */
    protected final static class ControlPanelProxyForAverageTemperature extends Proxy {
        private ControlPanelProxyForAverageTemperature(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Returns the value of the <code>targetTemperature</code> source of this <code>ControlPanel</code> device
         * 
         * @return the value of the <code>targetTemperature</code> source
         */
        public java.lang.Float getTargetTemperature() throws InvocationException {
            return (java.lang.Float) callGetValue("targetTemperature");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    
    /**
     * Discover object that will exposes the <code>HVACSystem</code> devices to get their sources for the
     * <code>when provided AverageTemperature</code> interaction contract.
     * 
     * <pre>
     * device HVACSystem extends PhysicalDevice {
     * 	source status as HVACStatus;
     * 	action Regulate;
     * }
     * </pre>
     */
    protected final static class HVACSystemDiscovererForAverageTemperature {
        private Service serviceParent;
        
        private HVACSystemDiscovererForAverageTemperature(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private HVACSystemCompositeForAverageTemperature instantiateComposite() {
            return new HVACSystemCompositeForAverageTemperature(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>HVACSystem</code> devices
         * 
         * @return a {@link HVACSystemCompositeForAverageTemperature} object composed of all discoverable <code>HVACSystem</code>
         */
        public HVACSystemCompositeForAverageTemperature all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>HVACSystem</code> devices
         * 
         * @return a {@link HVACSystemProxyForAverageTemperature} object pointing to a random discoverable <code>HVACSystem</code> device
         */
        public HVACSystemProxyForAverageTemperature anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>HVACSystem</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link HVACSystemCompositeForAverageTemperature} object composed of all matching <code>HVACSystem</code> devices
         */
        public HVACSystemCompositeForAverageTemperature whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>HVACSystem</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link HVACSystemCompositeForAverageTemperature} object composed of all matching <code>HVACSystem</code> devices
         */
        public HVACSystemCompositeForAverageTemperature whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>HVACSystem</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link HVACSystemCompositeForAverageTemperature} object composed of all matching <code>HVACSystem</code> devices
         */
        public HVACSystemCompositeForAverageTemperature whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>HVACSystem</code> devices to get their sources for the
     * <code>when provided AverageTemperature</code> interaction contract.
     * 
     * <pre>
     * device HVACSystem extends PhysicalDevice {
     * 	source status as HVACStatus;
     * 	action Regulate;
     * }
     * </pre>
     */
    protected final static class HVACSystemCompositeForAverageTemperature extends fr.inria.diagen.core.service.composite.Composite<HVACSystemProxyForAverageTemperature> {
        private HVACSystemCompositeForAverageTemperature(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/HVACSystem/");
        }
        
        @Override
        protected HVACSystemProxyForAverageTemperature instantiateProxy(RemoteServiceInfo rsi) {
            return new HVACSystemProxyForAverageTemperature(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link HVACSystemCompositeForAverageTemperature}, filtered using the attribute <code>id</code>.
         */
        public HVACSystemCompositeForAverageTemperature andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link HVACSystemCompositeForAverageTemperature}, filtered using the attribute <code>location</code>.
         */
        public HVACSystemCompositeForAverageTemperature andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link HVACSystemCompositeForAverageTemperature}, filtered using the attribute <code>user</code>.
         */
        public HVACSystemCompositeForAverageTemperature andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
    }
    
    /**
     * A proxy to one <code>HVACSystem</code> device to get its sources for the
     * <code>when provided AverageTemperature</code> interaction contract.
     * 
     * <pre>
     * device HVACSystem extends PhysicalDevice {
     * 	source status as HVACStatus;
     * 	action Regulate;
     * }
     * </pre>
     */
    protected final static class HVACSystemProxyForAverageTemperature extends Proxy {
        private HVACSystemProxyForAverageTemperature(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Returns the value of the <code>status</code> source of this <code>HVACSystem</code> device
         * 
         * @return the value of the <code>status</code> source
         */
        public fr.inria.phoenix.diasuite.framework.datatype.hvacstatus.HVACStatus getStatus() throws InvocationException {
            return (fr.inria.phoenix.diasuite.framework.datatype.hvacstatus.HVACStatus) callGetValue("status");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover object for AverageTemperature
}
