package fr.inria.phoenix.diasuite.framework.context.hvacregulation;

import java.io.Serializable;

/**
 * An object to store a value published by the context <code>HVACRegulation</code>.
 *
 * <pre>
 * context HVACRegulation as TemperatureLevel indexed by location as String {
 * 	when provided AverageTemperature 
 * 	get targetTemperature from ControlPanel,
 * 		status from HVACSystem
 * 	maybe publish;
 * }
 * </pre>
 */
public final class HVACRegulationValue implements Serializable {
    private static final long serialVersionUID = 0;
    
    private fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel value;
    
    /**
     * Get the value of the context <code>HVACRegulation</code>
     * 
     * @return the value of the context <code>HVACRegulation</code>
     */
    public fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel value() {
        return value;
    }
    
    private HVACRegulationIndices indices;
    
    /**
     * Get the value of the indices of the context <code>HVACRegulation</code>
     * 
     * @return the value of the indices
     */
    public HVACRegulationIndices indices() {
        return indices;
    }
    
    public HVACRegulationValue(fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel value, HVACRegulationIndices indices) {
        this.value = value;
        this.indices = indices;
    }
    
    public HVACRegulationValue(fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel value,
            java.lang.String location) {
        this.value = value;
        this.indices = new HVACRegulationIndices(location);
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        result = prime * result + ((indices == null) ? 0 : indices.hashCode());
        return result;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        HVACRegulationValue other = (HVACRegulationValue) obj;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        if (indices == null) {
            if (other.indices != null)
                return false;
        } else if (!indices.equals(other.indices))
            return false;
        return true;
    }
}
