package fr.inria.phoenix.diasuite.framework.controller.hvaccontroller;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.diagen.core.service.composite.CompositeException;
import fr.inria.diagen.core.exception.InvocationException;
import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

import fr.inria.phoenix.diasuite.framework.context.hvacregulation.HVACRegulationValue;

/**
 * <pre>
 * controller HVACController {
 * 	when provided HVACRegulation do Regulate on HVACSystem;
 * }
 * </pre>
 */
@SuppressWarnings("all")
public abstract class AbstractHVACController extends Service {
    
    public AbstractHVACController(ServiceConfiguration serviceConfiguration) {
        super("/Controller/HVACController/", serviceConfiguration);
    }
    
    // Methods from the Service class
    @Override
    protected final void internalPostInitialize() {
        subscribeValue("hVACRegulation", "/Context/HVACRegulation/"); // subscribe to HVACRegulation context
        postInitialize();
    }
    
    @Override
    public final void valueReceived(java.util.Map<String, Object> properties, RemoteServiceInfo source, String eventName, Object value, Object... indexes) {
        if (eventName.equals("hVACRegulation") && source.isCompatible("/Context/HVACRegulation/")) {
            HVACRegulationValue hVACRegulationValue = new HVACRegulationValue((fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel) value,
                    (java.lang.String) indexes[0]);
            
            onHVACRegulation(hVACRegulationValue, new DiscoverForHVACRegulation());
        }
    }
    // End of methods from the Service class
    
    // Interaction contract implementation
    /**
     * This method is called when the <code>HVACRegulation</code> context publishes a value.
     * 
     * <pre>
     * when provided HVACRegulation do Regulate on HVACSystem;
     * </pre>
     * 
     * @param hVACRegulation the value of the <code>HVACRegulation</code> context.
     * @param discover a discover object to get context values and action methods
     */
    protected abstract void onHVACRegulation(HVACRegulationValue hVACRegulation, DiscoverForHVACRegulation discover);
    
    // End of interaction contract implementation
    
    // Discover object for HVACRegulation
    /**
     * An object to discover devices and contexts for the following interaction contract:
     * 
     * <code>
     * when provided HVACRegulation do Regulate on HVACSystem;
     * </code>
     */
    protected final class DiscoverForHVACRegulation {
        private final HVACSystemDiscovererForHVACRegulation hVACSystemDiscoverer = new HVACSystemDiscovererForHVACRegulation(AbstractHVACController.this);
        
        /**
         * @return a {@link HVACSystemDiscovererForHVACRegulation} object to discover <code>HVACSystem</code> devices
         */
        public HVACSystemDiscovererForHVACRegulation hVACSystems() {
            return hVACSystemDiscoverer;
        }
    }
    
    /**
     * Discover object that will exposes the <code>HVACSystem</code> devices to execute action on for the
     * <code>when provided HVACRegulation</code> interaction contract.
     * 
     * <pre>
     * device HVACSystem extends PhysicalDevice {
     * 	source status as HVACStatus;
     * 	action Regulate;
     * }
     * </pre>
     */
    protected final static class HVACSystemDiscovererForHVACRegulation {
        private Service serviceParent;
        
        private HVACSystemDiscovererForHVACRegulation(Service serviceParent) {
            super();
            this.serviceParent = serviceParent;
        }
        
        private HVACSystemCompositeForHVACRegulation instantiateComposite() {
            return new HVACSystemCompositeForHVACRegulation(serviceParent);
        }
        
        /**
         * Returns a composite of all accessible <code>HVACSystem</code> devices
         * 
         * @return a {@link HVACSystemCompositeForHVACRegulation} object composed of all discoverable <code>HVACSystem</code>
         */
        public HVACSystemCompositeForHVACRegulation all() {
            return instantiateComposite();
        }
        
        /**
         * Returns a proxy to one out of all accessible <code>HVACSystem</code> devices
         * 
         * @return a {@link HVACSystemProxyForHVACRegulation} object pointing to a random discoverable <code>HVACSystem</code> device
         */
        public HVACSystemProxyForHVACRegulation anyOne() {
            return all().anyOne();
        }
        
        /**
         * Returns a composite of all accessible <code>HVACSystem</code> devices whose attribute <code>id</code> matches a given value.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return a {@link HVACSystemCompositeForHVACRegulation} object composed of all matching <code>HVACSystem</code> devices
         */
        public HVACSystemCompositeForHVACRegulation whereId(java.lang.String id) throws CompositeException {
            return instantiateComposite().andId(id);
        }
        
        /**
         * Returns a composite of all accessible <code>HVACSystem</code> devices whose attribute <code>location</code> matches a given value.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return a {@link HVACSystemCompositeForHVACRegulation} object composed of all matching <code>HVACSystem</code> devices
         */
        public HVACSystemCompositeForHVACRegulation whereLocation(java.lang.String location) throws CompositeException {
            return instantiateComposite().andLocation(location);
        }
        
        /**
         * Returns a composite of all accessible <code>HVACSystem</code> devices whose attribute <code>user</code> matches a given value.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return a {@link HVACSystemCompositeForHVACRegulation} object composed of all matching <code>HVACSystem</code> devices
         */
        public HVACSystemCompositeForHVACRegulation whereUser(java.lang.String user) throws CompositeException {
            return instantiateComposite().andUser(user);
        }
    }
    
    /**
     * A composite of several <code>HVACSystem</code> devices to execute action on for the
     * <code>when provided HVACRegulation</code> interaction contract.
     * 
     * <pre>
     * device HVACSystem extends PhysicalDevice {
     * 	source status as HVACStatus;
     * 	action Regulate;
     * }
     * </pre>
     */
    protected final static class HVACSystemCompositeForHVACRegulation extends fr.inria.diagen.core.service.composite.Composite<HVACSystemProxyForHVACRegulation> {
        private HVACSystemCompositeForHVACRegulation(Service serviceParent) {
            super(serviceParent, "/Device/Device/PhysicalDevice/HVACSystem/");
        }
        
        @Override
        protected HVACSystemProxyForHVACRegulation instantiateProxy(RemoteServiceInfo rsi) {
            return new HVACSystemProxyForHVACRegulation(service, rsi);
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>id</code>.
         * 
         * @param id The <code>id<code> attribute value to match.
         * @return this {@link HVACSystemCompositeForHVACRegulation}, filtered using the attribute <code>id</code>.
         */
        public HVACSystemCompositeForHVACRegulation andId(java.lang.String id) throws CompositeException {
            filterByAttribute("id", id);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>location</code>.
         * 
         * @param location The <code>location<code> attribute value to match.
         * @return this {@link HVACSystemCompositeForHVACRegulation}, filtered using the attribute <code>location</code>.
         */
        public HVACSystemCompositeForHVACRegulation andLocation(java.lang.String location) throws CompositeException {
            filterByAttribute("location", location);
            return this;
        }
        
        /**
         * Returns this composite in which a filter was set to the attribute <code>user</code>.
         * 
         * @param user The <code>user<code> attribute value to match.
         * @return this {@link HVACSystemCompositeForHVACRegulation}, filtered using the attribute <code>user</code>.
         */
        public HVACSystemCompositeForHVACRegulation andUser(java.lang.String user) throws CompositeException {
            filterByAttribute("user", user);
            return this;
        }
        
        /**
         * Executes the <code>warmUp()</code> action's method on all devices of this composite.
         */
        public void warmUp() throws InvocationException {
            launchDiscovering();
            for (HVACSystemProxyForHVACRegulation proxy : proxies) {
                proxy.warmUp();
            }
        }
        
        /**
         * Executes the <code>coolDown()</code> action's method on all devices of this composite.
         */
        public void coolDown() throws InvocationException {
            launchDiscovering();
            for (HVACSystemProxyForHVACRegulation proxy : proxies) {
                proxy.coolDown();
            }
        }
        
        /**
         * Executes the <code>stop()</code> action's method on all devices of this composite.
         */
        public void stop() throws InvocationException {
            launchDiscovering();
            for (HVACSystemProxyForHVACRegulation proxy : proxies) {
                proxy.stop();
            }
        }
    }
    
    /**
     * A proxy to one <code>HVACSystem</code> device to execute action on for the
     * <code>when provided HVACRegulation</code> interaction contract.
     * 
     * <pre>
     * device HVACSystem extends PhysicalDevice {
     * 	source status as HVACStatus;
     * 	action Regulate;
     * }
     * </pre>
     */
    protected final static class HVACSystemProxyForHVACRegulation extends Proxy {
        private HVACSystemProxyForHVACRegulation(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Executes the <code>warmUp()</code> action's method on this device.
         */
        public void warmUp() throws InvocationException {
            callOrder("warmUp");
        }
        
        /**
         * Executes the <code>coolDown()</code> action's method on this device.
         */
        public void coolDown() throws InvocationException {
            callOrder("coolDown");
        }
        
        /**
         * Executes the <code>stop()</code> action's method on this device.
         */
        public void stop() throws InvocationException {
            callOrder("stop");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
    // End of discover object for HVACRegulation
}
