package fr.inria.phoenix.diasuite.framework.device.dimmablelight;

import java.io.Serializable;

import fr.inria.diagen.core.network.RemoteServiceInfo;
import fr.inria.diagen.core.service.local.Service;
import fr.inria.diagen.core.service.proxy.Proxy;

/**
 * An object to store a value published on the source <code>status</code> of the device <code>DimmableLight</code>.
 *
 * <pre>
 * device DimmableLight extends Light {
 * 	action VariationControl;
 * 	source level as Float;
 * }
 * </pre>
 */
public final class StatusFromDimmableLight implements Serializable {
    private static final long serialVersionUID = 0;
    
    private fr.inria.phoenix.diasuite.framework.datatype.onoffstatus.OnOffStatus value;
    
    /**
     * Get the value of the source <code>status</code>
     * 
     * @return the value of the source <code>status</code>
     */
    public fr.inria.phoenix.diasuite.framework.datatype.onoffstatus.OnOffStatus value() {
        return value;
    }
    
    public StatusFromDimmableLight(Service service, RemoteServiceInfo remoteServiceInfo, fr.inria.phoenix.diasuite.framework.datatype.onoffstatus.OnOffStatus value) {
        this.sender = new DimmableLightProxy(service, remoteServiceInfo);
        this.value = value;
    }
    
    @Override
    public int hashCode() {
        final int prime = 31;
        int result = 1;
        result = prime * result + ((value == null) ? 0 : value.hashCode());
        return result;
    }
    
    @Override
    public boolean equals(Object obj) {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        StatusFromDimmableLight other = (StatusFromDimmableLight) obj;
        if (value == null) {
            if (other.value != null)
                return false;
        } else if (!value.equals(other.value))
            return false;
        return true;
    }
    
    // Proxy to the sender, i.e., the device that published the source
    private DimmableLightProxy sender;
    
    /**
     * Get the sender of the source <code>status</code>. I.e., the <code>DimmableLight</code> device that published the source.
     * 
     * @return A proxy to the <code>DimmableLight</code> that triggered the source
     */
    public DimmableLightProxy sender() {
        return sender;
    }
    
    /**
     * A proxy to a <code>DimmableLight</code> that discloses subscription/unsubscription methods.
     */
    public class DimmableLightProxy extends Proxy {
        private DimmableLightProxy(Service service, RemoteServiceInfo remoteServiceInfo) {
            super(service, remoteServiceInfo);
        }
        
        /**
         * Subcribes to publication of source <code>status</code> from this <code>DimmableLight</code>.
         */
        public void subscribeStatus() {
            getService().getProcessor().subscribeValue(getService().getOutProperties(), new fr.inria.diagen.core.service.filter.SubscriptionFilter(this.getRemoteServiceInfo()), "status");
        }
        
        /**
         * Unsubcribes from publication of source <code>status</code> from this <code>DimmableLight</code>.
         */
        public void unsubscribeStatus() {
            getService().getProcessor().unsubscribeValue(getService().getOutProperties(), new fr.inria.diagen.core.service.filter.SubscriptionFilter(this.getRemoteServiceInfo()), "status");
        }
        
        /**
         * @return the value of the <code>id</code> attribute
         */
        public java.lang.String id() {
            return (java.lang.String) callGetValue("id");
        }
        
        /**
         * @return the value of the <code>location</code> attribute
         */
        public java.lang.String location() {
            return (java.lang.String) callGetValue("location");
        }
        
        /**
         * @return the value of the <code>user</code> attribute
         */
        public java.lang.String user() {
            return (java.lang.String) callGetValue("user");
        }
    }
}
