package fr.inria.phoenix.scenario.airconditionmanager.context;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.phoenix.diasuite.framework.context.averagetemperature.AbstractAverageTemperature;
import fr.inria.phoenix.diasuite.framework.context.averagetemperature.AverageTemperatureIndices;
import fr.inria.phoenix.diasuite.framework.context.averagetemperature.AverageTemperatureValue;
import fr.inria.phoenix.diasuite.framework.datatype.temperatureunit.TemperatureUnit;
import fr.inria.phoenix.diasuite.framework.device.temperaturesensor.TemperatureFromTemperatureSensor;

public class AverageTemperature extends AbstractAverageTemperature {

	public AverageTemperature(ServiceConfiguration serviceConfiguration) {
		super(serviceConfiguration);
	}

	@Override
	protected AverageTemperatureValue onTemperatureFromTemperatureSensor(
			TemperatureFromTemperatureSensor temperatureFromTemperatureSensor,
			DiscoverForTemperatureFromTemperatureSensor discover) {

		String location = temperatureFromTemperatureSensor.sender().location();

		TemperatureSensorCompositeForTemperatureFromTemperatureSensor temperatureSensorComposite = discover
				.temperatureSensors().whereLocation(location);

		int sensorCount = 0;
		Float temperatureSum = 0F;

		for (TemperatureSensorProxyForTemperatureFromTemperatureSensor temperatureProxy : temperatureSensorComposite) {
			temperatureSum = temperatureSum
					+ temperatureProxy.getTemperature(TemperatureUnit.CELSIUS);
			sensorCount++;
		}

		Float averageTemperature = temperatureSum / sensorCount;

		return new AverageTemperatureValue(averageTemperature,
				new AverageTemperatureIndices(location));
	}

}
