package fr.inria.phoenix.scenario.airconditionmanager.context;

import fr.inria.diagen.core.ServiceConfiguration;
import fr.inria.phoenix.diasuite.framework.context.averagetemperature.AverageTemperatureValue;
import fr.inria.phoenix.diasuite.framework.context.hvacregulation.AbstractHVACRegulation;
import fr.inria.phoenix.diasuite.framework.datatype.hvacstatus.HVACStatus;
import fr.inria.phoenix.diasuite.framework.datatype.temperaturelevel.TemperatureLevel;

public class HVACRegulation extends AbstractHVACRegulation {

	public HVACRegulation(ServiceConfiguration serviceConfiguration) {
		super(serviceConfiguration);
	}

	@Override
	protected HVACRegulationValuePublishable onAverageTemperature(
			AverageTemperatureValue averageTemperatureValue,
			DiscoverForAverageTemperature discover) {

		// average temperature
		Float averageTemperature = averageTemperatureValue.value();

		// etage
		String location = averageTemperatureValue.indices().location();

		// control panel
		ControlPanelProxyForAverageTemperature controlPanel = discover
				.controlPanels().whereLocation(location).anyOne();

		// control panel target temperature
		Float targetTemperature = controlPanel.getTargetTemperature();

		// hvac system pour l'etage
		HVACSystemProxyForAverageTemperature hvacSystem = discover
				.hVACSystems().whereLocation(location).anyOne();

		// status de hvac system
		HVACStatus hvacSystemStatus = hvacSystem.getStatus();

		if ((Float.compare(averageTemperature, targetTemperature) > 0)
				&& !hvacSystemStatus.equals(HVACStatus.COOLING))
			return new HVACRegulationValuePublishable(TemperatureLevel.HOT,
					location, true);

		if ((Float.compare(averageTemperature, targetTemperature) < 0)
				&& !hvacSystemStatus.equals(HVACStatus.HEATING))
			return new HVACRegulationValuePublishable(TemperatureLevel.COLD,
					location, true);

		if ((Float.compare(averageTemperature, targetTemperature) == 0)
				&& !hvacSystemStatus.equals(HVACStatus.STOPPED))
			return new HVACRegulationValuePublishable(TemperatureLevel.OK,
					location, true);

		return new HVACRegulationValuePublishable(TemperatureLevel.OK,
				location, false);

	}

}
