import static fr.inria.phoenix.diasuite.framework.mocks.Mock.TIMEOUT;
import static fr.inria.phoenix.diasuite.framework.mocks.Mock.mockControlPanel;
import static fr.inria.phoenix.diasuite.framework.mocks.Mock.mockHVACSystem;
import static fr.inria.phoenix.diasuite.framework.mocks.Mock.mockTemperatureSensor;
import static fr.inria.phoenix.diasuite.framework.mocks.Mock.shutdown;
import static fr.inria.phoenix.diasuite.framework.mocks.Mock.underTest;
import static org.junit.Assert.assertTrue;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import fr.inria.phoenix.diasuite.framework.datatype.hvacstatus.HVACStatus;
import fr.inria.phoenix.diasuite.framework.datatype.temperatureunit.TemperatureUnit;
import fr.inria.phoenix.diasuite.framework.mocks.ControlPanelMock;
import fr.inria.phoenix.diasuite.framework.mocks.HVACSystemMock;
import fr.inria.phoenix.diasuite.framework.mocks.TemperatureSensorMock;
import fr.inria.phoenix.scenario.airconditionmanager.ComponentBinder;


public class HVACTestCase {

	private TemperatureSensorMock temperatureSensor;
	private ControlPanelMock controlPanel;
	private HVACSystemMock hvacSystem;

	@Before
	public void setUp() throws Exception {
		TIMEOUT = 1000;
		underTest(ComponentBinder.class);
	}

	@After
	public void tearDown() throws Exception {
		shutdown();
	}

	@Test
	public void testWarmUpLocation() {
		temperatureSensor = mockTemperatureSensor("TemperatureSensor", "A29",
				"Tester");
		controlPanel = mockControlPanel("ControlPanel", "A29", "Tester");
		hvacSystem = mockHVACSystem("HVACSystem", "A29", "Tester");

		controlPanel.setTargetTemperature(15.0f);
		hvacSystem.setStatus(HVACStatus.STOPPED);
		temperatureSensor.temperature(10.0f, TemperatureUnit.CELSIUS);

		assertTrue(hvacSystem.expectWarmUp());
	}

	@Test
	public void testCoolDownLocation() {
		temperatureSensor = mockTemperatureSensor("TemperatureSensor", "A30",
				"Tester");
		controlPanel = mockControlPanel("ControlPanel", "A30", "Tester");
		hvacSystem = mockHVACSystem("HVACSystem", "A30", "Tester");

		controlPanel.setTargetTemperature(10.0f);
		hvacSystem.setStatus(HVACStatus.HEATING);
		temperatureSensor.temperature(15.0f, TemperatureUnit.CELSIUS);

		assertTrue(hvacSystem.expectCoolDown());
	}

	@Test
	public void testDoNothingInLocation() {
		temperatureSensor = mockTemperatureSensor("TemperatureSensor", "A31",
				"Tester");
		controlPanel = mockControlPanel("ControlPanel", "A31", "Tester");
		hvacSystem = mockHVACSystem("HVACSystem", "A31", "Tester");

		controlPanel.setTargetTemperature(20.0f);
		hvacSystem.setStatus(HVACStatus.COOLING);
		temperatureSensor.temperature(20.0f, TemperatureUnit.CELSIUS);

		assertTrue(hvacSystem.expectStop());
	}

}
